@@ -45,12 +45,14 @@
 #'   `py_conf` packages are requested, the `RETICULATE_PYTHON` environment
 #'   variable is set to ensure the Nix environment does not use a system-wide
 #'   Python installation. If you are developing a Python package, set
-#'   `py_src_dir` to the path of your package's source directory (e.g.,
 #'   `"mypackage/src"` or just `"src"`). This adds `PYTHONPATH` to the shell
 #'   hook so your package can be imported without installation. This is the Nix
 #'   equivalent of `pip install -e .` (editable install). Note: if `"uv"` is in
 #'   `system_pkgs`, `LD_LIBRARY_PATH` is automatically configured for dynamic
-#'   library loading (required by packages like numpy).
+#'   library loading (required by packages like numpy). You can also install packages
+#'   from git or PyPI by adding `git_pkgs` (list of lists with `package_name`, `repo_url`, `commit`)
+#'   and `pypi_pkgs` (vector of package names, optionally with version `name@version`) to
+#'   the `py_conf` list.
 #' @param jl_conf List. A list of two elements, `jl_version` and `jl_conf`.
 #'   `jl_version` must be of the form `"1.10"` for Julia 1.10. Leave empty or
 #'   use an empty string to use the latest version, or use `"lts"` for the long
@@ -123,12 +125,13 @@
 #'   set `ide = "none"` and refer to the `vignette("configuring-ide")` for
 #'   more details on how to set up your editor to work with Nix shells.
 #'
-#'   Packages to install from GitHub or Gitlab must be provided in a list of 3
-#'   elements: "package_name", "repo_url" and "commit". To install several
-#'   packages, provide a list of lists of these 3 elements, one per package to
-#'   install. It is also possible to install old versions of packages by
-#'   specifying a version. For example, to install the latest version of `{AER}`
-#'   but an old version of `{ggplot2}`, you could write: `r_pkgs = c("AER",
+#'   Packages to install from GitHub, GitLab, or other custom Git hosts
+#'   (Forgejo, Gitea, cgit, etc.) must be provided in a list of 3 elements:
+#'   "package_name", "repo_url" and "commit". To install several packages,
+#'   provide a list of lists of these 3 elements, one per package to install. It
+#'   is also possible to install old versions of packages by specifying a
+#'   version. For example, to install the latest version of `{AER}` but an old
+#'   version of `{ggplot2}`, you could write: `r_pkgs = c("AER",
 #'   "ggplot2@2.2.1")`. Note however that doing this could result in dependency
 #'   hell, because an older version of a package might need older versions of
 #'   its dependencies, but other packages might need more recent versions of the
@@ -137,15 +140,15 @@
 #'   the Nix revision closest to that date, by setting `r_ver = "3.1.0"`, which
 #'   was the version of R current at the time. This ensures that Nix builds a
 #'   completely coherent environment. For security purposes, users that wish to
-#'   install packages from GitHub/GitLab or from the CRAN archives must provide
-#'   a security hash for each package. `{rix}` automatically precomputes this
-#'   hash for the source directory of R packages from GitHub/Gitlab or from the
-#'   CRAN archives, to make sure the expected trusted sources that match the
-#'   precomputed hashes in the `default.nix` are downloaded, but only if Nix
-#'   is installed. If you need to generate an expression with such packages,
-#'   but are working on a system where you can't install Nix, consider generating
-#'   the expression using a continuous integration service, such as GitHub
-#'   Actions.
+#'   install packages from GitHub, GitLab, or other Git hosts, or from the CRAN
+#'   archives must provide a security hash for each package. `{rix}`
+#'   automatically precomputes this hash for the source directory of R packages
+#'   from Git hosts or from the CRAN archives, to make sure the expected trusted
+#'   sources that match the precomputed hashes in the `default.nix` are
+#'   downloaded, but only if Nix is installed. If you need to generate an
+#'   expression with such packages, but are working on a system where you can't
+#'   install Nix, consider generating the expression using a continuous
+#'   integration service, such as GitHub Actions.
 #'
 #'   Note that installing packages from Git or old versions using the `"@"`
 #'   notation or local packages, does not leverage Nix's capabilities for
@@ -380,11 +383,11 @@ for more details."
   flag_git_archive <- if (
     !is.null(git_pkgs) || !is.null(cran_pkgs$archive_pkgs)
   ) {
-    # If git_pkgs is a list of lists, then sapply will succeed
+    # If git_pkgs is a list of lists, then vapply will succeed
     # if not, then we can access "package_name" directly
     git_pkgs_names <- if (!is.null(git_pkgs)) {
       tryCatch(
-        sapply(git_pkgs, function(x) x$package_name),
+        vapply(git_pkgs, function(x) x$package_name, character(1)),
         error = function(e) git_pkgs$package_name
       )
     }
@@ -394,7 +397,7 @@ for more details."
     # which will be the name of the package
     cran_archive_names <- if (!is.null(cran_pkgs$archive_pkgs)) {
       pkgs <- strsplit(cran_pkgs$archive_pkgs, split = "@")
-      sapply(pkgs, function(x) x[[1]])
+      vapply(pkgs, function(x) x[[1]], character(1))
     }
 
     paste0(c(git_pkgs_names, cran_archive_names), collapse = " ")
